@@ -19,7 +19,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
-#include <Python.h>
+#include <sbkpython.h>
 #include "pysidesignal.h"
 #include "pysidesignal_p.h"
 #include "signalmanager.h"
@@ -75,8 +75,7 @@ static PyMethodDef Signal_methods[] = {
 };
 
 PyTypeObject PySideSignalMetaType = {
-    PyObject_HEAD_INIT(0)
-    /*ob_size*/             0,
+    PyVarObject_HEAD_INIT(0, 0)
     /*tp_name*/             "PySide.QtCore.MetaSignal",
     /*tp_basicsize*/        sizeof(PyTypeObject),
     /*tp_itemsize*/         0,
@@ -110,9 +109,7 @@ PyTypeObject PySideSignalMetaType = {
 };
 
 PyTypeObject PySideSignalType = {
-    PyObject_HEAD_INIT(&PySideSignalMetaType)
-    //PyObject_HEAD_INIT(0)
-    /*ob_size*/             0,
+    PyVarObject_HEAD_INIT(&PySideSignalMetaType, 0)
     /*tp_name*/             "PySide.QtCore."SIGNAL_CLASS_NAME,
     /*tp_basicsize*/        sizeof(PySideSignal),
     /*tp_itemsize*/         0,
@@ -174,8 +171,7 @@ static PyMappingMethods SignalInstance_as_mapping = {
 };
 
 PyTypeObject PySideSignalInstanceType = {
-    PyObject_HEAD_INIT(0)
-    /*ob_size*/             0,
+    PyVarObject_HEAD_INIT(0, 0)
     /*tp_name*/             "PySide.QtCore."SIGNAL_INSTANCE_NAME,
     /*tp_basicsize*/        sizeof(PySideSignalInstance),
     /*tp_itemsize*/         0,
@@ -244,7 +240,7 @@ int signalTpInit(PyObject* self, PyObject* args, PyObject* kwds)
 
     for(Py_ssize_t i = 0, i_max = PyTuple_Size(args); i < i_max; i++) {
         PyObject *arg = PyTuple_GET_ITEM(args, i);
-        if (PySequence_Check(arg) && !PyString_Check(arg)) {
+        if (PySequence_Check(arg) && !Shiboken::String::check(arg)) {
             tupledArgs = true;
             PySide::Signal::appendSignature(data, PySide::Signal::parseSignature(arg));
         }
@@ -290,7 +286,7 @@ PyObject* signalGetItem(PyObject* self, PyObject* key)
     }
     char* sig = PySide::Signal::buildSignature(data->signalName, sigKey);
     free(sigKey);
-    PyObject* pySignature = PyString_FromString(sig);
+    PyObject* pySignature = Shiboken::String::fromCString(sig);
     free(sig);
     return pySignature;
 }
@@ -374,9 +370,14 @@ PyObject* signalInstanceConnect(PyObject* self, PyObject* args, PyObject* kwds)
     if (match) {
         Shiboken::AutoDecRef tupleArgs(PyList_AsTuple(pyArgs));
         Shiboken::AutoDecRef pyMethod(PyObject_GetAttrString(source->d->source, "connect"));
-        return PyObject_CallObject(pyMethod, tupleArgs);
+        PyObject* result = PyObject_CallObject(pyMethod, tupleArgs);
+        if (result == Py_True)
+            return result;
+        else
+            Py_XDECREF(result);
     }
-
+    if (!PyErr_Occurred())
+        PyErr_Format(PyExc_RuntimeError, "Fail to connect signal %s.", source->d->signature);
     return 0;
 }
 
@@ -416,7 +417,6 @@ PyObject* signalInstanceGetItem(PyObject* self, PyObject* key)
     }
     PyErr_Format(PyExc_IndexError, "Signature %s not found for signal: %s", sig, sigName);
     free(sig);
-
     return 0;
 }
 
@@ -460,9 +460,14 @@ PyObject* signalInstanceDisconnect(PyObject* self, PyObject* args)
     if (match) {
         Shiboken::AutoDecRef tupleArgs(PyList_AsTuple(pyArgs));
         Shiboken::AutoDecRef pyMethod(PyObject_GetAttrString(source->d->source, "disconnect"));
-        return PyObject_CallObject(pyMethod, tupleArgs);
+        PyObject* result = PyObject_CallObject(pyMethod, tupleArgs);
+        if (result == Py_True)
+            return result;
+        else
+            Py_DECREF(result);
     }
 
+    PyErr_Format(PyExc_RuntimeError, "Fail to disconnect signal %s.", source->d->signature);
     return 0;
 }
 
@@ -564,7 +569,7 @@ char* getTypeName(PyObject* type)
         } else {
             // Translate python types to Qt names
             PyTypeObject *objType = reinterpret_cast<PyTypeObject*>(type);
-            if ((objType == &PyString_Type)  ||  (objType == &PyUnicode_Type))
+            if (Shiboken::String::checkType(objType))
                 typeName = strdup("QString");
             else if (objType == &PyInt_Type)
                 typeName = strdup("int");
@@ -574,16 +579,16 @@ char* getTypeName(PyObject* type)
                 typeName = strdup("double");
             else if (objType == &PyBool_Type)
                 typeName = strdup("bool");
-            else if (objType->ob_type == &SbkEnumType_Type)
+            else if (Py_TYPE(objType) == &SbkEnumType_Type)
                 typeName = strdup(Shiboken::Enum::getCppName(objType));
             else
                 typeName = strdup("PyObject");
         }
         return typeName;
-    } else if (PyString_Check(type)) {
-        return strdup(PyString_AS_STRING(type));
-    } else if (type == Py_None) {
+    } else if (type == Py_None) { // Must be checked before as Shiboken::String::check accepts Py_None
         return strdup("void");
+    } else if (Shiboken::String::check(type)) {
+        return strdup(Shiboken::String::toCString(type));
     }
     return 0;
 }
@@ -600,7 +605,7 @@ char* buildSignature(const char *name, const char *signature)
 char* parseSignature(PyObject *args)
 {
     char *signature = 0;
-    if (args && (PyString_Check(args) || !PySequence_Check(args)))
+    if (args && (Shiboken::String::check(args) || !PySequence_Check(args)))
         return getTypeName(args);
 
     for(Py_ssize_t i = 0, i_max = PySequence_Size(args); i < i_max; i++) {
@@ -647,7 +652,7 @@ void instanceInitialize(PySideSignalInstance* self, PyObject* name, PySideSignal
     if (data->signalName)
         selfPvt->signalName = strdup(data->signalName);
     else {
-        selfPvt->signalName = strdup(PyString_AsString(name));
+        selfPvt->signalName = strdup(Shiboken::String::toCString(name));
         data->signalName = strdup(selfPvt->signalName);
     }
 
@@ -672,9 +677,15 @@ bool connect(PyObject* source, const char* signal, PyObject* callback)
     if (pyMethod.isNull())
         return false;
 
-    Shiboken::AutoDecRef pySignature(PyString_FromString(signal));
+    Shiboken::AutoDecRef pySignature(Shiboken::String::fromCString(signal));
     Shiboken::AutoDecRef pyArgs(PyTuple_Pack(3, source, pySignature.object(), callback));
-    return PyObject_CallObject(pyMethod, pyArgs);
+    PyObject* result =  PyObject_CallObject(pyMethod, pyArgs);
+    if (result == Py_False) {
+        PyErr_Format(PyExc_RuntimeError, "Fail to connect signal %s, to python callable object.", signal);
+        Py_DECREF(result);
+        result = 0;
+    }
+    return result;
 }
 
 PySideSignalInstance* newObjectFromMethod(PyObject* source, const QList<QMetaMethod>& methodList)
@@ -762,17 +773,24 @@ static void _addSignalToWrapper(SbkObjectType* wrapperType, const char* signalNa
     PyDict_SetItemString(typeDict, signalName, reinterpret_cast<PyObject*>(signal));
 }
 
+// This function is used by qStableSort to promote empty signatures
+static bool compareSignals(const QByteArray& sig1, const QByteArray& sig2)
+{
+    return sig1.isEmpty();
+}
+
 void registerSignals(SbkObjectType* pyObj, const QMetaObject* metaObject)
 {
     typedef QHash<QByteArray, QList<QByteArray> > SignalSigMap;
     SignalSigMap signalsFound;
     for(int i = metaObject->methodOffset(), max = metaObject->methodCount(); i < max; ++i) {
         QMetaMethod method = metaObject->method(i);
-        QByteArray methodName(method.signature());
-        methodName.chop(methodName.size() - methodName.indexOf('('));
 
-        if (method.methodType() == QMetaMethod::Signal)
+        if (method.methodType() == QMetaMethod::Signal) {
+            QByteArray methodName(method.signature());
+            methodName.chop(methodName.size() - methodName.indexOf('('));
             signalsFound[methodName] << join(method.parameterTypes(), ",");
+        }
     }
 
     SignalSigMap::Iterator it = signalsFound.begin();
@@ -785,7 +803,8 @@ void registerSignals(SbkObjectType* pyObj, const QMetaObject* metaObject)
         self->initialized = 0;
         self->homonymousMethod = 0;
 
-        qSort(it.value().begin(), it.value().end());
+        // Empty signatures comes first! So they will be the default signal signature
+        qStableSort(it.value().begin(), it.value().end(), &compareSignals);
         SignalSigMap::mapped_type::const_iterator j = it.value().begin();
         SignalSigMap::mapped_type::const_iterator endJ = it.value().end();
         for (; j != endJ; ++j)
@@ -801,7 +820,7 @@ PyObject* buildQtCompatible(const char* signature)
     QByteArray ba;
     ba.append(QT_SIGNAL_SENTINEL);
     ba.append(signature);
-    return PyString_FromStringAndSize(ba, ba.size());
+    return Shiboken::String::fromStringAndSize(ba, ba.size());
 }
 
 void addSignalToWrapper(SbkObjectType* wrapperType, const char* signalName, PySideSignal* signal)
@@ -859,7 +878,7 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
     if (isMethod || isFunction) {
         PyObject* function = isMethod ? PyMethod_GET_FUNCTION(callback) : callback;
         PyCodeObject* objCode = reinterpret_cast<PyCodeObject*>(PyFunction_GET_CODE(function));
-        functionName = PyString_AS_STRING(reinterpret_cast<PyFunctionObject*>(function)->func_name);
+        functionName = Shiboken::String::toCString(reinterpret_cast<PyFunctionObject*>(function)->func_name);
         useSelf = isMethod;
         numArgs = objCode->co_flags & CO_VARARGS ? -1 : objCode->co_argcount;
     } else if (PyCFunction_Check(callback)) {
