@@ -50,15 +50,28 @@
 
 namespace {
     static PyObject *metaObjectAttr = 0;
+
+    static int callMethod(QObject* object, int id, void** args);
+    static PyObject* parseArguments(QList<QByteArray> paramTypese, void** args);
+    static bool emitShortCircuitSignal(QObject* source, int signalIndex, PyObject* args);
+    static bool emitNormalSignal(QObject* source, int signalIndex, const char* signal, PyObject* args, const QStringList& argTypes);
+
+#ifdef IS_PY3K
+    static void destroyMetaObject(PyObject* obj)
+    {
+        delete reinterpret_cast<PySide::DynamicQMetaObject*>(PyCapsule_GetPointer(obj, 0));
+    }
+
+#else
     static void destroyMetaObject(void* obj)
     {
         delete reinterpret_cast<PySide::DynamicQMetaObject*>(obj);
     }
+#endif
 }
 
 namespace PySide {
 
-static int callMethod(QObject* object, int id, void** args);
 
 PyObjectWrapper::PyObjectWrapper()
     :m_me(Py_None)
@@ -113,9 +126,15 @@ QDataStream &operator<<(QDataStream& out, const PyObjectWrapper& myObj)
     }
     Shiboken::AutoDecRef repr(PyObject_CallFunctionObjArgs(reduce_func, (PyObject*)myObj, NULL));
     if (repr.object()) {
-        char* buff;
-        Py_ssize_t size;
-        PyString_AsStringAndSize(repr.object(), &buff, &size);
+        const char* buff = 0;
+        Py_ssize_t size  = 0;
+        if (PyBytes_Check(repr.object())) {
+            buff = PyBytes_AS_STRING(repr.object());
+            size = PyBytes_GET_SIZE(repr.object());
+        } else if (Shiboken::String::check(repr.object())) {
+            buff = Shiboken::String::toCString(repr.object());
+            size = Shiboken::String::len(repr.object());
+        }
         QByteArray data(buff, size);
         out << data;
     }
@@ -139,7 +158,7 @@ QDataStream &operator>>(QDataStream& in, PyObjectWrapper& myObj)
 
     QByteArray repr;
     in >> repr;
-    Shiboken::AutoDecRef pyCode(PyString_FromStringAndSize(repr.data(), repr.size()));
+    Shiboken::AutoDecRef pyCode(PyBytes_FromStringAndSize(repr.data(), repr.size()));
     Shiboken::AutoDecRef value(PyObject_CallFunctionObjArgs(eval_func, pyCode.object(), 0));
     if (!value.object()) {
         value = Py_None;
@@ -220,7 +239,7 @@ SignalManager::SignalManager() : m_d(new SignalManagerPrivate)
     PySide::registerCleanupFunction(clearSignalManager);
 
     if (!metaObjectAttr)
-        metaObjectAttr = PyString_FromString("__METAOBJECT__");
+        metaObjectAttr = Shiboken::String::fromCString("__METAOBJECT__");
 }
 
 void SignalManager::clear()
@@ -270,10 +289,12 @@ QObject* SignalManager::globalReceiver(QObject *sender, PyObject *callback)
     SharedMap globalReceivers = m_d->m_globalReceivers;
     QByteArray hash = GlobalReceiverV2::hash(callback);
     GlobalReceiverV2* gr = 0;
-    if (!globalReceivers->contains(hash) && sender) {
+    if (!globalReceivers->contains(hash)) {
         gr = (*globalReceivers)[hash] = new GlobalReceiverV2(callback, globalReceivers);
-        gr->incRef(sender); // create a link reference
-        gr->decRef(); // remove extra reference
+        if (sender) {
+            gr->incRef(sender); // create a link reference
+            gr->decRef(); // remove extra reference
+        }
     } else {
         gr = (*globalReceivers)[hash];
         if (sender)
@@ -309,6 +330,241 @@ int SignalManager::globalReceiverSlotIndex(QObject* receiver, const char* signat
     return reinterpret_cast<GlobalReceiverV2*>(receiver)->addSlot(signature);
 }
 
+bool SignalManager::emitSignal(QObject* source, const char* signal, PyObject* args)
+{
+    if (!Signal::checkQtSignal(signal))
+        return false;
+    signal++;
+
+    int signalIndex = source->metaObject()->indexOfSignal(signal);
+    if (signalIndex != -1) {
+        bool isShortCircuit;
+        QStringList argTypes = Signal::getArgsFromSignature(signal, &isShortCircuit);
+
+        if (isShortCircuit)
+            return emitShortCircuitSignal(source, signalIndex, args);
+        else
+            return emitNormalSignal(source, signalIndex, signal, args, argTypes);
+    }
+    return false;
+}
+
+int SignalManager::qt_metacall(QObject* object, QMetaObject::Call call, int id, void** args)
+{
+    const QMetaObject* metaObject = object->metaObject();
+    PySideProperty* pp = 0;
+    PyObject* pp_name = 0;
+    QMetaProperty mp;
+    PyObject* pySelf = 0;
+    int methodCount = metaObject->methodCount();
+    int propertyCount = metaObject->propertyCount();
+
+    if (call != QMetaObject::InvokeMetaMethod) {
+        mp = metaObject->property(id);
+        if (!mp.isValid())
+            return id - methodCount;
+
+        Shiboken::GilState gil;
+        pySelf = (PyObject*)Shiboken::BindingManager::instance().retrieveWrapper(object);
+        Q_ASSERT(pySelf);
+        pp_name = Shiboken::String::fromCString(mp.name());
+        pp = Property::getObject(pySelf, pp_name);
+        if (!pp) {
+            qWarning("Invalid property: %s.", mp.name());
+            Py_XDECREF(pp_name);
+            return id - methodCount;
+        }
+    }
+
+    switch(call) {
+#ifndef QT_NO_PROPERTIES
+        case QMetaObject::ReadProperty:
+        case QMetaObject::WriteProperty:
+        case QMetaObject::ResetProperty:
+        case QMetaObject::QueryPropertyDesignable:
+        case QMetaObject::QueryPropertyScriptable:
+        case QMetaObject::QueryPropertyStored:
+        case QMetaObject::QueryPropertyEditable:
+        case QMetaObject::QueryPropertyUser:
+            pp->d->metaCallHandler(pp, pySelf, call, args);
+            break;
+#endif
+        case QMetaObject::InvokeMetaMethod:
+            id = callMethod(object, id, args);
+            break;
+
+        default:
+            qWarning("Unsupported meta invocation type.");
+    }
+
+    // WARNING Isn't safe to call any metaObject and/or object methods beyond this point
+    //         because the object can be deleted inside the called slot.
+
+    if (call == QMetaObject::InvokeMetaMethod)
+        id = id - methodCount;
+    else
+        id = id - propertyCount;
+
+    if (pp || pp_name) {
+        Shiboken::GilState gil;
+        Py_XDECREF(pp);
+        Py_XDECREF(pp_name);
+    }
+    return id;
+}
+
+int SignalManager::callPythonMetaMethod(const QMetaMethod& method, void** args, PyObject* pyMethod, bool isShortCuit)
+{
+    Q_ASSERT(pyMethod);
+
+    Shiboken::GilState gil;
+    PyObject* pyArguments = NULL;
+
+    if (isShortCuit)
+        pyArguments = reinterpret_cast<PyObject*>(args[1]);
+    else
+        pyArguments = parseArguments(method.parameterTypes(), args);
+
+    //keep the returnType this call be destroyed after method call
+    QByteArray returnType = method.typeName();
+
+    Shiboken::AutoDecRef retval(PyObject_CallObject(pyMethod, pyArguments));
+
+    if (!isShortCuit)
+        Py_XDECREF(pyArguments);
+
+    if (retval.isNull()) {
+        PyErr_Print();
+    } else {
+        if (returnType.size() > 0)
+            Shiboken::TypeResolver::get(returnType)->toCpp(retval, &args[0]);
+    }
+
+    return -1;
+}
+
+bool SignalManager::registerMetaMethod(QObject* source, const char* signature, QMetaMethod::MethodType type)
+{
+    int ret = registerMetaMethodGetIndex(source, signature, type);
+    return (ret != -1);
+}
+
+int SignalManager::registerMetaMethodGetIndex(QObject* source, const char* signature, QMetaMethod::MethodType type)
+{
+    Q_ASSERT(source);
+    const QMetaObject* metaObject = source->metaObject();
+    int methodIndex = metaObject->indexOfMethod(signature);
+    // Create the dynamic signal is needed
+    if (methodIndex == -1) {
+        SbkObject* self = Shiboken::BindingManager::instance().retrieveWrapper(source);
+        if (!Shiboken::Object::hasCppWrapper(self)) {
+            qWarning() << "Invalid Signal signature:" << signature;
+            return -1;
+        } else {
+            DynamicQMetaObject *dmo = 0;
+            PyObject *pySelf = reinterpret_cast<PyObject*>(self);
+            PyObject* dict = self->ob_dict;
+
+            // Create a instance meta object
+            if (!dict || !PyDict_Contains(dict, metaObjectAttr)) {
+                dmo = new DynamicQMetaObject(pySelf->ob_type, metaObject);
+#ifdef IS_PY3K
+                PyObject *pyDmo = PyCapsule_New(dmo, 0, destroyMetaObject);
+#else
+                PyObject *pyDmo = PyCObject_FromVoidPtr(dmo, destroyMetaObject);
+#endif
+
+                PyObject_SetAttr(pySelf, metaObjectAttr, pyDmo);
+                Py_DECREF(pyDmo);
+            } else {
+                dmo = reinterpret_cast<DynamicQMetaObject*>(const_cast<QMetaObject*>(metaObject));
+            }
+
+            if (type == QMetaMethod::Signal)
+                return dmo->addSignal(signature);
+            else
+                return dmo->addSlot(signature);
+        }
+    }
+    return methodIndex;
+}
+
+bool SignalManager::hasConnectionWith(const QObject *object)
+{
+    return m_d->m_globalReceiver.hasConnectionWith(object);
+}
+
+const QMetaObject* SignalManager::retriveMetaObject(PyObject *self)
+{
+    Shiboken::GilState gil;
+    DynamicQMetaObject *mo = 0;
+    Q_ASSERT(self);
+
+    PyObject* dict = reinterpret_cast<SbkObject*>(self)->ob_dict;
+    if (dict && PyDict_Contains(dict, metaObjectAttr)) {
+        PyObject *pyMo = PyDict_GetItem(dict, metaObjectAttr);
+
+#ifdef IS_PY3K
+        mo = reinterpret_cast<DynamicQMetaObject*>(PyCapsule_GetPointer(pyMo, 0));
+#else
+        mo = reinterpret_cast<DynamicQMetaObject*>(PyCObject_AsVoidPtr(pyMo));
+#endif
+    } else {
+        mo = reinterpret_cast<DynamicQMetaObject*>(Shiboken::Object::getTypeUserData(reinterpret_cast<SbkObject*>(self)));
+    }
+
+    mo->update();
+    return mo;
+}
+
+namespace {
+
+static int callMethod(QObject* object, int id, void** args)
+{
+    const QMetaObject* metaObject = object->metaObject();
+    QMetaMethod method = metaObject->method(id);
+
+    if (method.methodType() == QMetaMethod::Signal) {
+        // emit python signal
+        QMetaObject::activate(object, id, args);
+    } else {
+        Shiboken::GilState gil;
+        PyObject* self = (PyObject*)Shiboken::BindingManager::instance().retrieveWrapper(object);
+        QByteArray methodName = method.signature();
+        methodName = methodName.left(methodName.indexOf('('));
+        Shiboken::AutoDecRef pyMethod(PyObject_GetAttrString(self, methodName));
+        SignalManager::callPythonMetaMethod(method, args, pyMethod, false);
+    }
+    return -1;
+}
+
+
+static PyObject* parseArguments(QList<QByteArray> paramTypes, void** args)
+{
+    PyObject* preparedArgs = NULL;
+    Py_ssize_t argsSize = paramTypes.count();
+
+    if (argsSize)
+        preparedArgs = PyTuple_New(argsSize);
+
+    for (int i = 0, max = paramTypes.count(); i < max; ++i) {
+        void* data = args[i+1];
+        const char* dataType = paramTypes[i].constData();
+
+        Shiboken::TypeResolver* tr = Shiboken::TypeResolver::get(dataType);
+        if (tr) {
+            PyObject* arg = tr->toPython(data);
+            PyTuple_SET_ITEM(preparedArgs, i, arg);
+        } else {
+            PyErr_Format(PyExc_TypeError, "Can't call meta function because I have no idea how to handle %s", dataType);
+            Py_DECREF(preparedArgs);
+            return NULL;
+        }
+    }
+
+    return preparedArgs;
+}
+
 static bool emitShortCircuitSignal(QObject* source, int signalIndex, PyObject* args)
 {
     void* signalArgs[2] = {0, args};
@@ -361,206 +617,4 @@ static bool emitNormalSignal(QObject* source, int signalIndex, const char* signa
     return ok;
 }
 
-bool SignalManager::emitSignal(QObject* source, const char* signal, PyObject* args)
-{
-    if (!Signal::checkQtSignal(signal))
-        return false;
-    signal++;
-
-    int signalIndex = source->metaObject()->indexOfSignal(signal);
-    if (signalIndex != -1) {
-        bool isShortCircuit;
-        QStringList argTypes = Signal::getArgsFromSignature(signal, &isShortCircuit);
-
-        if (isShortCircuit)
-            return emitShortCircuitSignal(source, signalIndex, args);
-        else
-            return emitNormalSignal(source, signalIndex, signal, args, argTypes);
-    }
-    return false;
-}
-
-int SignalManager::qt_metacall(QObject* object, QMetaObject::Call call, int id, void** args)
-{
-    const QMetaObject* metaObject = object->metaObject();
-    PySideProperty* pp = 0;
-    PyObject* pp_name = 0;
-    QMetaProperty mp;
-    PyObject* pySelf = 0;
-    int methodCount = metaObject->methodCount();
-    int propertyCount = metaObject->propertyCount();
-
-    if (call != QMetaObject::InvokeMetaMethod) {
-        mp = metaObject->property(id);
-        if (!mp.isValid())
-            return id - methodCount;
-
-        Shiboken::GilState gil;
-        pySelf = (PyObject*)Shiboken::BindingManager::instance().retrieveWrapper(object);
-        Q_ASSERT(pySelf);
-        pp_name = PyString_FromString(mp.name());
-        pp = Property::getObject(pySelf, pp_name);
-        if (!pp) {
-            qWarning("Invalid property: %s.", mp.name());
-            Py_XDECREF(pp_name);
-            return id - methodCount;
-        }
-    }
-
-    switch(call) {
-#ifndef QT_NO_PROPERTIES
-        case QMetaObject::ReadProperty:
-        case QMetaObject::WriteProperty:
-        case QMetaObject::ResetProperty:
-        case QMetaObject::QueryPropertyDesignable:
-        case QMetaObject::QueryPropertyScriptable:
-        case QMetaObject::QueryPropertyStored:
-        case QMetaObject::QueryPropertyEditable:
-        case QMetaObject::QueryPropertyUser:
-            pp->d->metaCallHandler(pp, pySelf, call, args);
-            break;
-#endif
-        case QMetaObject::InvokeMetaMethod:
-            id = callMethod(object, id, args);
-            break;
-
-        default:
-            qWarning("Unsupported meta invocation type.");
-    }
-
-    // WARNING Isn't safe to call any metaObject and/or object methods beyond this point
-    //         because the object can be deleted inside the called slot.
-
-    if (call == QMetaObject::InvokeMetaMethod)
-        id = id - methodCount;
-    else
-        id = id - propertyCount;
-
-    if (pp || pp_name) {
-        Shiboken::GilState gil;
-        Py_XDECREF(pp);
-        Py_XDECREF(pp_name);
-    }
-    return id;
-}
-
-static int PySide::callMethod(QObject* object, int id, void** args)
-{
-    const QMetaObject* metaObject = object->metaObject();
-    QMetaMethod method = metaObject->method(id);
-
-    if (method.methodType() == QMetaMethod::Signal) {
-        // emit python signal
-        QMetaObject::activate(object, id, args);
-    } else {
-        // call python slot
-        Shiboken::GilState gil;
-        QList<QByteArray> paramTypes = method.parameterTypes();
-        PyObject* self = (PyObject*)Shiboken::BindingManager::instance().retrieveWrapper(object);
-        PyObject* preparedArgs = NULL;
-        Py_ssize_t argsSize = paramTypes.count();
-
-        if (argsSize)
-            preparedArgs = PyTuple_New(argsSize);
-
-        for (int i = 0, max = paramTypes.count(); i < max; ++i) {
-            void* data = args[i+1];
-            const char* dataType = paramTypes[i].constData();
-
-            Shiboken::TypeResolver* tr = Shiboken::TypeResolver::get(dataType);
-            if (tr) {
-                PyObject* arg = tr->toPython(data);
-                PyTuple_SET_ITEM(preparedArgs, i, arg);
-            } else {
-                PyErr_Format(PyExc_TypeError, "Can't call meta function because I have no idea how to handle %s", dataType);
-                return -1;
-            }
-        }
-
-        QString methodName = method.signature();
-        methodName = methodName.left(methodName.indexOf('('));
-
-        Shiboken::AutoDecRef pyMethod(PyObject_GetAttrString(self, qPrintable(methodName)));
-        if (!pyMethod.isNull()) {
-            Shiboken::AutoDecRef retval(PyObject_CallObject(pyMethod, preparedArgs));
-            if (retval.isNull()) {
-                qWarning() << "Error calling slot" << methodName;
-                PyErr_Print();
-            } else {
-                const char* returnType = method.typeName();
-                if (returnType && (strlen(returnType) > 0))
-                    Shiboken::TypeResolver::get(returnType)->toCpp(retval, &args[0]);
-            }
-        } else {
-            qWarning() << "Dynamic slot" << methodName << "not found!";
-        }
-        Py_XDECREF(preparedArgs);
-    }
-    return -1;
-}
-bool SignalManager::registerMetaMethod(QObject* source, const char* signature, QMetaMethod::MethodType type)
-{
-    int ret = registerMetaMethodGetIndex(source, signature, type);
-    return (ret != -1);
-}
-
-int SignalManager::registerMetaMethodGetIndex(QObject* source, const char* signature, QMetaMethod::MethodType type)
-{
-    Q_ASSERT(source);
-    const QMetaObject* metaObject = source->metaObject();
-    int methodIndex = metaObject->indexOfMethod(signature);
-    // Create the dynamic signal is needed
-    if (methodIndex == -1) {
-        SbkObject* self = Shiboken::BindingManager::instance().retrieveWrapper(source);
-        if (!Shiboken::Object::hasCppWrapper(self)) {
-            qWarning() << "Invalid Signal signature:" << signature;
-            return -1;
-        } else {
-            DynamicQMetaObject *dmo = 0;
-            PyObject *pySelf = reinterpret_cast<PyObject*>(self);
-            PyObject* dict = self->ob_dict;
-
-            // Create a instance meta object
-            if (!dict || !PyDict_Contains(dict, metaObjectAttr)) {
-                dmo = new DynamicQMetaObject(pySelf->ob_type, metaObject);
-                PyObject *pyDmo = PyCObject_FromVoidPtr(dmo, destroyMetaObject);
-                PyObject_SetAttr(pySelf, metaObjectAttr, pyDmo);
-                Py_DECREF(pyDmo);
-            } else {
-                dmo = reinterpret_cast<DynamicQMetaObject*>(const_cast<QMetaObject*>(metaObject));
-            }
-
-            if (type == QMetaMethod::Signal)
-                return dmo->addSignal(signature);
-            else
-                return dmo->addSlot(signature);
-        }
-    }
-    return methodIndex;
-}
-
-bool SignalManager::hasConnectionWith(const QObject *object)
-{
-    return m_d->m_globalReceiver.hasConnectionWith(object);
-}
-
-
-const QMetaObject* SignalManager::retriveMetaObject(PyObject *self)
-{
-    Shiboken::GilState gil;
-    DynamicQMetaObject *mo = 0;
-    Q_ASSERT(self);
-
-    PyObject* dict = reinterpret_cast<SbkObject*>(self)->ob_dict;
-    if (dict && PyDict_Contains(dict, metaObjectAttr)) {
-        PyObject *pyMo = PyDict_GetItem(dict, metaObjectAttr);
-        mo = reinterpret_cast<DynamicQMetaObject*>(PyCObject_AsVoidPtr(pyMo));
-    } else {
-        mo = reinterpret_cast<DynamicQMetaObject*>(Shiboken::Object::getTypeUserData(reinterpret_cast<SbkObject*>(self)));
-    }
-
-    mo->update();
-    return mo;
-}
-
-
+} //namespace
